# -*- coding: utf-8 -*-
'''stuf has attributes'''

from stuf.core import defaultstuf, fixedstuf, frozenstuf, orderedstuf, stuf

__all__ = ('defaultstuf', 'fixedstuf', 'frozenstuf', 'orderedstuf', 'stuf')
__version__ = (0, 8, 9)


idefaultstuf = defaultstuf
ifixedstuf = fixedstuf
ifrozenstuf = frozenstuf
iorderedstuf = orderedstuf
istuf = stuf
